      program benchmark
c
c      This program solves the heat equation in 2-space dimensions
c      by the Alternating Direction Implicit method.
c      Parameters are -
c      u  vector of values for the odd traverses
c      v  same for the even traverses
c      ucoef  matrix of coefficients for the odd traverses
c      vcoef  matrix of traverses for the even traverses
c      bcndu   vector of boundary values for ucoeff values
c      bcndv   same for vcoeff
c      top  vector of boundary values across the top
c      bot  same for bottom of region
c      lft  same for left hand edge
c      rt  same for right hand edge
c      m  number of rows of node points
c      n  number of columns of node points
c      diff Diffusion coefficient
c      h the value of delta x=delta y
c      stime step size,related to delta x through r
c      tmax  maximum value of stime for which values are required
c
c      Set up the vectors and matrices


      real ucoef(500,3),vcoef(500,3),bcndu(500),bcndv(500)
      real top(100),bot(100),lft(100),rt(100),u(500),v(500)
      logical*4 ok

c      Initialize some values with data statements

      data ucoef,vcoef/3000*-1.0/
      data m,n,r/7,15,0.5/
      data h,diff/0.125,0.152/
      data tmax/0.0/
      data rt/100*100.00/
      data lft/100*0.0/
      data top/100*0.0/
      data bot/100*0.0/
      data bcndu,bcndv/1000*0.0/
c      For the test case we begin with temperatures everywhere equal
c      to zero
c      Establish these by a data statement
      data v/500*0.0/
c      write(6,*)'Input modelled drift time (between 1 & 5000 secs):'
c      read(5,*)tmax
c      write(6,*)'Drift time confirmed (secs) : ',tmax
      do ijk = 1 , 200
      tmax = 200.0
      if (ijk.eq.1) write(6,*)'Working ...'

c      Establish the coefficient matrices by over-writing on the
c      diagonal and certain off-diagonal terms.

      msize=n*m
      do 10 i=1,msize
            ucoef(i,2)=1.0/r+2.0
            vcoef(i,2)=1.0/r+2.0
10      continue

      ml1=m-1
      nl1=n-1
      msl1=msize-1
      do 20 i=n,msl1,n
            ucoef(i,3)=0.0
            ucoef(i+1,1)=0.0
20      continue

      do 30 i=m,msl1,m
            vcoef(i,3)=0.0
            vcoef(i+1,1)=0.0
30      continue

c      Now get values into the bcond vectors
      do 40 i=1,n
            bcndu(i)=top(i)
            j=msize-n+i
            bcndu(j)=bot(i)
40      continue
      do 45 i=1,m
            j=(i-1)*n+1
            bcndu(j)=bcndu(j)+lft(i)
            j=i*n
            bcndu(j)=bcndu(j)+rt(i)
45      continue

      do 50 i=1,m
            bcndv(i)=lft(i)
            j=msize-m+i
            bcndv(j)=rt(i)
50      continue
      do 55 i=1,n
            j=(i-1)*m+1
            bcndv(j)=bcndv(j)+top(i)
            j=i*m
            bcndv(j)=bcndv(j)+bot(i)
55      continue
c      We now set the LU decompositions of the two coefficient matrices
c      These are very easy to compute. We store them back in the original
c      vector space
      do 60 i=2,msize
            ucoef(i-1,3)=ucoef(i-1,3)/ucoef(i-1,2)
            ucoef(i,2)=ucoef(i,2)-ucoef(i,1)*ucoef(i-1,3)
            vcoef(i-1,3)=vcoef(i-1,3)/vcoef(i-1,2)
            vcoef(i,2)=vcoef(i,2)-vcoef(i,1)*vcoef(i-1,3)
60      continue
c      Now we begin the iterations until stime equals tmax
      dt=r/diff*h*h
      stime=0.0   
      kkk = 0
62      if(stime.gt.tmax) goto 1000
      kkk = kkk + 1
c      compute r.h.s. for the u equations
c      store these in the u vector
c      First do the top and bottom sets of terms
      do 65 i=1,n
            j=(i-1)*m+1
            u(i)=(1.0/r-2.0)*v(j)+v(j+1)+bcndu(i)
            k=msize-n+i
            j=i*m
            u(k)=v(j-1)+(1.0/r-2.0)*v(j)+bcndu(k)
65      continue
c      Now do the intermediate ones
      do 75 i=2,ml1
            do 70 j=1,n
                  k=(i-1)*n+j
                  l=i+(j-1)*m
                  u(k)=v(l-1)+(1.0/r-2.0)*v(l)+v(l+1)+bcndu(k)
70            continue
75      continue
c      Now get solution for the horizontal traverse
      u(1)=u(1)/ucoef(1,2)
      do 80 i=2,msize
            u(i)=(u(i)-ucoef(i,1)*u(i-1))/ucoef(i,2)
80      continue
      do 90 i=1,msl1
            jrow=msize-i
            u(jrow)=u(jrow)-ucoef(jrow,3)*u(jrow+1)
90      continue

c      We do the same for the vertical traverse- u and v exchange roles
c      Compute r.h.s. for the v equations ,store in v.
      do 95 i=1,m
      j=(i-1)*n+1
      v(i)=(1.0/r-2.0)*u(j)+u(j+1)+bcndv(i)
      k=msize-m+i
      j=i*n
      v(k)=u(j-1)+(1.0/r-2.0)*u(j)+bcndv(k)
95      continue
c      Do the intermediate rows

      do 105 i=2,nl1
            do 100 j=1,m
                  k=(i-1)*m+j
                  l=i+(j-1)*n
                  v(k)=u(l-1)+(1.0/r-2.0)*u(l)+u(l+1)+bcndv(k)
100            continue
105      continue

c      Get the solution for the odd traverse.

      v(1)=v(1)/vcoef(1,2)
      do 110 i=2,msize
      v(i)=(v(i)-vcoef(i,1)*v(i-1))/vcoef(i,2)
110      continue
      do 120 i=1,msl1
      jrow=msize-i
      v(jrow)=v(jrow)-vcoef(jrow,3)*v(jrow+1)
120      continue
      stime=stime+2.0*dt
      goto 62
      
1000  print *, kkk , 'Iterations'
      open(10,file='bench.out',status='unknown')
      write(10,*)' When t= ',stime,' the solution vector is :'

      do 130 i=1,msize
         write(10,202)i,v(i)
 130      continue
 202         format(1x,'v(',i4,') =',1x,f8.2)

      if ( ijk.eq.1)write(6,*)'Drift time at completion (secs) : ',stime
      close(10)
      enddo
      end
