C *** Large matrix multiplication benchmark, double-precision
C *** David L. Staples
      PROGRAM largmat8
      IMPLICIT NONE
      INTEGER*4 ibig
      PARAMETER (ibig = 500)
      INTEGER*4 i,j
      REAL*8 a(ibig,ibig),b(ibig,ibig),c(ibig,ibig)
      REAL*4 time,time1,time2,time3,time4,time5
      REAL*4 secnds
 
c      OPEN (7,FILE = 'summary.dat',STATUS = 'unknown',position= 'append'
c     & )
c      WRITE (7,*) 'LARGMAT8'
      print * , 'LM8 Output ================================='
      PRINT *,'Double-Precision Large Matrix Multiplication Benchmark'
      PRINT *,'    Matrix Order =',ibig
 
      time = secnds(0.0)
      DO i = 1,ibig
         DO j = 1,ibig
            a(i,j) = dexp(dsin(dble(i-1)/57.295))+dlog(dble(j))
            b(j,i) = datan2(dlog(dsqrt(a(i,j)**4)),dtan(a(i,j)))
         END DO
      END DO
      time1 = secnds(time)
      write(*,100) time1
 
      time = secnds(0.0)
      CALL mat_mult_for3(c,ibig,a,ibig,b,ibig,ibig,ibig,ibig)
      time2 = secnds(time)
      write(*,200) time2
 
      time = secnds(0.0)
      CALL mat_mult_for1(c,ibig,a,ibig,b,ibig,ibig,ibig,ibig)
      time3 = secnds(time)
      write(*,300) time3
 
      time = secnds(0.0)
      CALL mat_mult_for05(c,a,b,ibig,ibig,ibig)
      time4 = secnds(time)
      write(*,400) time4
 
      time = secnds(0.0)
      CALL mat_mult_for0(c,a,b,ibig,ibig,ibig)
      time5 = secnds(time)
      write(*,500) time5
 
      time = time1+time2+time3+time4+time5
      write(*,600) time
c      WRITE (7,*) time
  100 FORMAT (F11.4,' seconds for setting up matrices')
  200 FORMAT (F11.4,' seconds for variable dimensions')
  300 FORMAT (F11.4,' seconds for variable dimensions, hand optimized')
  400 FORMAT (F11.4,' seconds for constant dimensions')
  500 FORMAT (F11.4,' seconds for constant dimensions, hand optimized')
  600 FORMAT (F11.4,' seconds total.'/)
      END
 
      SUBROUTINE mat_mult_for3(c,nc,a,na,b,nb,n1,n2,n3)
      IMPLICIT NONE
      INTEGER*4 ibig
      PARAMETER (ibig = 150)
      INTEGER na,nb,nc,n1,n2,n3
      INTEGER*4 i,j,k
      REAL*8 c(nc,n3),a(na,n2),b(nb,n3)
 
      DO k = 1,n3
         DO i = 1,n1
            c(i,k) = 0.
            DO j = 1,n2
               c(i,k) = c(i,k)+a(i,j)*b(j,k)
            END DO
         END DO
      END DO
      RETURN
      END
 
C     THIS VERSION HAS THE INVARIANT MATRIX ELEMENT MOVED OUT OF THE LOOP
C     MACTUTOR CALLS THIS 'HAND OPTIMIZING'
      SUBROUTINE mat_mult_for1(c,nc,a,na,b,nb,n1,n2,n3)
      IMPLICIT NONE
      INTEGER*4 ibig
      PARAMETER (ibig = 150)
      INTEGER na,nb,nc,n1,n2,n3
      INTEGER*4 i,j,k
      REAL*8 c(nc,n3),a(na,n2),b(nb,n3)
      REAL*8 sum
 
      DO k = 1,n3
         DO i = 1,n1
            sum = 0.
            DO j = 1,n2
               sum = sum+a(i,j)*b(j,k)
            END DO
            c(i,k) = sum
         END DO
      END DO
      RETURN
      END
 
C     THIS VERSION HAS CONSTANT DIMENSIONS BUT NOT 'HAND OPTIMIZED'
C     (MACTUTOR SHOULD HAVE COMPARED THIS ONE TO PASCAL)
      SUBROUTINE mat_mult_for05(c,a,b,n1,n2,n3)
      IMPLICIT NONE
      INTEGER*4 ibig
      PARAMETER (ibig = 150)
      INTEGER n1,n2,n3
      INTEGER*4 i,j,k
      REAL*8 c(ibig,ibig),a(ibig,ibig),b(ibig,ibig)
 
      DO k = 1,n3
         DO i = 1,n1
            c(i,k) = 0.
            DO j = 1,n2
               c(i,k) = c(i,k)+a(i,j)*b(j,k)
            END DO
         END DO
      END DO
      RETURN
      END
 
C     THIS VERSION HAS CONSTANT DIMENSIONS AND 'HAND OPTIMIZED'
C     (MACTUTOR COMPARES THIS TO PASCAL, HAND-OPTIMIZED)
      SUBROUTINE mat_mult_for0(c,a,b,n1,n2,n3)
      IMPLICIT NONE
      INTEGER n1,n2,n3
      INTEGER*4 ibig
      PARAMETER (ibig = 150)
      INTEGER*4 i,j,k
      REAL*8 c(ibig,ibig),a(ibig,ibig),b(ibig,ibig)
      REAL*8 sum
 
      DO k = 1,n3
         DO i = 1,n1
            sum = 0.
            DO j = 1,n2
               sum = sum+a(i,j)*b(j,k)
            END DO
            c(i,k) = sum
         END DO
      END DO
      RETURN
      END
C     INCLUDE 'timer.for'
