C *** Complex root-finding benchmark
C *** David L. Staples
 
      PROGRAM xzroots
      INTEGER m, m1
      PARAMETER (m = 5*10-1, m1 = m+1)
      INTEGER i
      COMPLEX a(m1), roots(m)
      LOGICAL polish
      REAL*4  secnds, time
      DATA a/(0.0,2.0), (0.2,0.3), (-1.5,-2.0), (0.1,0.1), (1.2,0.2),
     1       (1.0,2.0), (0.1,0.6), (-1.0,-2.5), (7.0,3.2), (6.3,3.1),
     1       (2.1,3.4), (1.2,0.7), (-2.3,-2.3), (4.1,2.4), (1.4,1.8),
     1       (2.3,3.1), (3.3,1.8), (-5.3,-2.3), (6.5,4.2), (3.7,7.8),
     1       (2.4,3.8), (4.2,2.9), (-6.3,-2.4), (5.6,5.3), (4.8,6.8),
     1       (2.0,3.0), (0.2,3.7), (-5.3,-3.3), (5.7,2.4), (7.4,1.8),
     1       (2.3,3.6), (3.2,4.7), (-2.3,-4.3), (6.3,4.4), (3.4,5.8),
     1       (2.4,3.7), (4.2,1.7), (-4.3,-5.3), (5.4,5.4), (4.4,6.8),
     1       (2.0,3.0), (0.2,0.7), (-5.3,-2.3), (5.5,2.4), (7.4,1.8),
     1       (3.0,4.0), (0.4,0.2), (-1.0,-2.0), (4.0,1.5), (1.5,0.9)/
      print*, ' RO Output ==================================='
      print*, 'ROOTS complex root-finder test...'
      time = secnds(0.0)
      polish = .FALSE.
      CALL zroots(a, m, roots, polish)
c     write (*,'(/1x,a)') 'Unpolished roots:'
c     write (*,'(1x,t10,a,t25,a,t37,a)') 'Root #', 'Real', 'Imag.'
c     DO i = 1, m
c        write (*,'(1x,i11,5x,2f12.6)') i, roots(i)
c     END DO
c     write (*,'(/1x,a)') 'Corrupted roots:'
      DO itimes = 1, 5000
         DO i = 1, m
            roots(i) = roots(i)*(1.0+0.01*i)
         END DO
c        write (*,'(1x,t10,a,t25,a,t37,a)') 'Root #', 'Real', 'Imag.'
c        DO i = 1, m
c           write (*,'(1x,i11,5x,2f12.6)') i, roots(i)
c        END DO
         polish = .TRUE.
         CALL zroots(a, m, roots, polish)
      END DO
      time = secnds(time)
c      open (7, file='summary.dat', status='unknown',position='append')
c      write (7,*) 'ROOTS'
c      write (7,*) time
      write (*,'(/1x,a)') 'Polished roots:'
      write (*,'(1x,t10,a,t25,a,t37,a)') 'Root #', 'Real', 'Imag.'
      DO i = 1, m
        write (*,'(1x,i11,5x,2f12.6)') i, roots(i)
      END DO
      print*, '   Time = ', time
      END
 
 
      SUBROUTINE zroots(a, m, roots, polish)
      INTEGER m, maxm
      REAL eps
      COMPLEX a(m+1), roots(m)
      LOGICAL polish
      PARAMETER (eps = 1.E-6, maxm = 101)
C     U    USES LAGUER
      INTEGER i, j, jj, its
      COMPLEX ad(maxm), x, b, c
      DO j = 1, m+1
         ad(j) = a(j)
      END DO
      DO j = m, 1, -1
         x = cmplx(0., 0.)
         CALL laguer(ad, j, x, its)
         IF ( abs(aimag(x)) .LE. 2.*eps**2*abs(real(x)) ) x = cmplx
     &    (real(x), 0.)
         roots(j) = x
         b = ad(j+1)
         DO jj = j, 1, -1
            c = ad(jj)
            ad(jj) = b
            b = x*b+c
         END DO
      END DO
      IF ( polish ) THEN
         DO j = 1, m
            CALL laguer(a, m, roots(j), its)
         END DO
      END IF
      DO j = 2, m
         x = roots(j)
         DO i = j-1, 1, -1
            IF ( real(roots(i)) .LE. real(x) ) GO TO 1
            roots(i+1) = roots(i)
         END DO
         i = 0
    1    roots(i+1) = x
      END DO
      RETURN
      END
 
 
      SUBROUTINE laguer(a, m, x, its)
      INTEGER m, its, maxit, mr, mt
      REAL epss
      COMPLEX a(m+1), x
      PARAMETER (epss = 2.E-7, mr = 8, mt = 10, maxit = mt*mr)
      INTEGER iter, j
      REAL abx, abp, abm, err, frac(mr)
      COMPLEX dx, x1, b, d, f, g, h, sq, gp, gm, g2
      SAVE frac
      DATA frac/.5, .25, .75, .13, .38, .62, .88, 1./
      DO iter = 1, maxit
         its = iter
         b = a(m+1)
         err = abs(b)
         d = cmplx(0., 0.)
         f = cmplx(0., 0.)
         abx = abs(x)
         DO j = m, 1, -1
            f = x*f+d
            d = x*d+b
            b = x*b+a(j)
            err = abs(b)+abx*err
         END DO
         err = epss*err
         IF ( abs(b) .LE. err ) RETURN
         g = d/b
         g2 = g*g
         h = g2-2.*f/b
         sq = sqrt((m-1)*(m*h-g2))
         gp = g+sq
         gm = g-sq
         abp = abs(gp)
         abm = abs(gm)
         IF ( abp .LT. abm ) gp = gm
         IF ( max(abp, abm) .GT. 0. ) THEN
            dx = m/gp
         ELSE
            dx = exp(cmplx(log(1.+abx), real(iter)))
         END IF
         x1 = x-dx
         IF ( x .EQ. x1 ) RETURN
         IF ( mod(iter, mt) .EQ. 0 ) THEN
            x = x-dx*frac(iter/mt)
         ELSE
            x = x1
         END IF
      END DO
      PAUSE'too many iterations in laguer'
      RETURN
      END
C     INCLUDE 'timer.for'
