c     Single-precision Whetstone.
c     Slightly modified from a Microsoft Fortran demo code.
 
      INTEGER*4 j, k, l, i, isave
      INTEGER*4 n2, n3, n4, n6, n7, n8, n9, n11
      INTEGER*4 inner, outer, kount, npass, max_pass
 
      REAL*4 x, y, z, t1, t2, t3, e1(4)
      REAL*8 whet_save, dif_save, kilowhet
      REAL*4 begin_time, end_time, dif_time
      REAL*8 error, whet_err, percent_err
      REAL*4 secnds
 
      COMMON t1, t2, t3, e1, j, k, l
 
c        open (7, file='summary.dat', status='unknown',position='append')
c        write (7,*) 'WHET4'
      print*, 'WH4 Output - Single-precision Whetstone test...'
 
c     initialize pass-control variables.  whet4 must make at least
c     two passes to calculate sensitivity terms.  npass counts passes;
c     max_pass is the maximum number of passes.  currently the program
c     is written such that max_pass should have a value of 2.  for higher
c     values of max_pass, modifications to the program are required.
 
      npass = 0
      max_pass = 2
 
c     WRITE (*, 100)
 
 
c 100 FORMAT ('0Number of inner loops (suggest more than 3):  '\)
c     READ (*, *) inner
c     WRITE (*, 200)
c 200 FORMAT (' Number of outer loops (suggest more than 1):  '\)
c     READ (*, *) outer
 
      inner = 500
      outer = 100
 
      DO WHILE ( npass .LT. max_pass )
          WRITE (*, 300) npass+1, outer, inner
  300     FORMAT (/' Pass #', I3.2, ': ', I10, ' outer loop(s),', I10,
     &     ' inner loop(s)')
          WRITE (*, *) ('=', j = 1, 60)
          kount = 0
          begin_time = secnds(0.0)
 
 
c         begininning of timed interval
 
          DO WHILE ( kount .LT. outer )
 
c             whetstone code begins here.  first initialize variables
c             and loop counters based on the number of inner loops.
 
c             loops 2 and 3 (described below) use variations of the
c             following transformation statements:
 
c             x1 = ( x1 + x2 + x3 - x4) * 0.5
c             x2 = ( x1 + x2 - x3 + x4) * 0.5
c             x3 = ( x1 - x2 + x3 + x4) * 0.5
c             x4 = (-x1 + x2 + x3 + x4) * 0.5
 
c             theoretically this set tends to the solution
 
c             x1 = x2 = x3 = x4 = 1.0
 
c             the variables t1, t2, and t3 are terms designed to limit
c             convergence of the set.
 
              t1 = 0.499975
              t2 = 0.50025
              t3 = 2.0
 
 
c             the variables n2-n11 are counters for loops 2-11.
c             based on earlier statistical work (wichmann, 1970),
c             loops 1, 5, and 10 are omitted from the program.
 
              isave = inner
              n2 = 12*inner
              n3 = 14*inner
              n4 = 345*inner
              n6 = 210*inner
              n7 = 32*inner
              n8 = 899*inner
              n9 = 616*inner
              n11 = 93*inner
 
c             the values in array e1 are arbitrary.
 
              e1(1) = 1.0
              e1(2) = -1.0
              e1(3) = -1.0
              e1(4) = -1.0
 
c             loop 1 - convergence test using real numbers.  the
c             execution of this loop was found to be statistically
c             invalid, but is included here for completeness.
 
c             do i = 1, n1
c             x1 = ( x1 + x2 + x3 - x4) * t1
c             x2 = ( x1 + x2 - x3 + x4) * t1
c             x3 = ( x1 - x2 + x3 + x4) * t1
c             x4 = (-x1 + x2 + x3 + x4) * t1
c             end do
 
c             loop 2 - convergence test using array elements.
 
              DO i = 1, n2
                  e1(1) = (e1(1)+e1(2)+e1(3)-e1(4))*t1
                  e1(2) = (e1(1)+e1(2)-e1(3)+e1(4))*t1
                  e1(3) = (e1(1)-e1(2)+e1(3)+e1(4))*t1
                  e1(4) = (-e1(1)+e1(2)+e1(3)+e1(4))*t1
              END DO
 
 
c             loop 3 - convergence test using subroutine calls.
 
              DO i = 1, n3
                  CALL sub1(e1)
              END DO
 
c             loop 4 - conditional jumps.  repeated iterations
c             alternate the value of j between 0 and 1.
 
              j = 1
              DO i = 1, n4
                  IF ( j .EQ. 1 ) THEN
                      j = 2
                  ELSE
                      j = 3
                  END IF
                  IF ( j .GT. 2 ) THEN
                      j = 0
                  ELSE
                      j = 1
                  END IF
                  IF ( j .LT. 1 ) THEN
                      j = 1
                  ELSE
                      j = 0
                  END IF
              END DO
 
c             loop 6 - integer arithmetic and array addressing.
c             the values of integers j, k, and l remain unchanged
c             through iterations of loop.
 
              j = 1
              k = 2
              l = 3
              DO i = 1, n6
                  j = j*(k-j)*(l-k)
                  k = l*k-(l-j)*k
                  l = (l-k)*(k+j)
                  e1(l-1) = j+k+l
                  e1(k-1) = j*k*l
              END DO
 
c             loop 7 - trigonometric functions.  the following loop
c             almost transforms x and y into themselves and produces
c             results that slowly vary.  (the value of t1 ensures
c             slow convergence, as described above.)
 
              x = 0.5
              y = 0.5
              DO i = 1, n7
                  x = t1*atan(t3*sin(x)*cos(x)/(cos(x+y)+cos(x-y)-1.0))
                  y = t1*atan(t3*sin(y)*cos(y)/(cos(x+y)+cos(x-y)-1.0))
              END DO
 
c             loop 8 - subroutine calls.  values of x, y, and z
c             are arbitrary.
 
              x = 1.0
              y = 1.0
              z = 1.0
              DO i = 1, n8
                  CALL sub2(x, y, z)
              END DO
 
c             loop 9 - array references and subroutine calls.
 
              j = 1
              k = 2
              l = 3
              e1(1) = 1.0
              e1(2) = 2.0
              e1(3) = 3.0
              DO i = 1, n9
                  CALL sub3()
              END DO
 
c             loop 10 - simple integer arithmetic.  the execution
c             of this loop was found to be statistically invalid,
c             but is included here for completeness.
 
c             j = 2
c             k = 3
c             do i = 1, n10
c             j = j + k
c             k = j + k
c             j = j - k
c             k = k - j - j
c             end do
 
 
c             loop 11 - standard functions sqrt, exp, and alog.
 
              x = 0.75
              DO i = 1, n11
                  x = sqrt(exp(alog(x)/t2))
              END DO
 
c             end of whetstone code.
 
              inner = isave
              kount = kount+1
          END DO
 
c         end of timed interval
 
          end_time = secnds(0.0)
          dif_time = end_time-begin_time
 
c         1000 whetstones (kilowhetstones) = 100 * loops per second
 
          kilowhet = 100.0D00*dble(outer*inner)/dif_time
 
          WRITE (*, 400) dif_time, kilowhet
  400     FORMAT (' Elapsed time =', F12.2, ' seconds'/
     &     ' Whetstones   =', F12.2,
     &     ' single-precision kilowhets/second')
 
c         repeat with inner count doubled.
 
          npass = npass+1
          IF ( npass .LT. max_pass ) THEN
              dif_save = dif_time
              whet_save = kilowhet
              inner = inner*max_pass
          END IF
      END DO
 
 
c     compute sensitivity.
 
      error = dif_time-(dif_save*max_pass)
      whet_err = whet_save-kilowhet
      percent_err = whet_err*100.0D00/kilowhet
        avwhet = (whet_save + kilowhet) / 2.
      WRITE (*, *)
      WRITE (*, *) ('=', j = 1, 60)
      WRITE (*, 500) error, whet_err, percent_err, avwhet
  500 FORMAT (' Time error   =', F12.2, ' seconds'/' Whet error   =',
     & F12.2, ' kwhets/sec'/' %    error   =', F12.2, ' % whet error'/
     & ' Average      =',f12.2,' REAL*4 kwhets/sec'/)
      IF ( dif_time .LT. 10.0D00 ) WRITE (*, *)
     & 'TIME is less than 10 seconds -- ', 'suggest larger inner loop'
c      write (7,*) avwhet
 
      END
 
 
 
      SUBROUTINE sub1(e)
 
      REAL*4 t1, t2, t3, e(4)
c     common t1, t2, t3
      COMMON t1, t2, t3, e1, j, k, l
 
      DO i = 1, 6
          e(1) = (e(1)+e(2)+e(3)-e(4))*t1
          e(2) = (e(1)+e(2)-e(3)+e(4))*t1
          e(3) = (e(1)-e(2)+e(3)+e(4))*t1
          e(4) = (-e(1)+e(2)+e(3)+e(4))/t3
      END DO
      RETURN
      END
 
 
 
      SUBROUTINE sub2(x, y, z)
 
      REAL*4 t1, t2, t3, x1, y1, x, y, z
c     common t1, t2, t3
      COMMON t1, t2, t3, e1, j, k, l
 
      x1 = x
      y1 = y
      x1 = (x1+y1)*t1
      y1 = (x1+y1)*t1
      z = (x1+y1)/t3
      RETURN
      END
 
 
 
      SUBROUTINE sub3
 
      REAL*4 t1, t2, t3, e1(4)
      COMMON t1, t2, t3, e1, j, k, l
 
      e1(j) = e1(k)
      e1(k) = e1(l)
      e1(l) = e1(j)
      RETURN
      END
C     INCLUDE 'timer.for'
